package javax.management.remote.rmi;

import sun.reflect.misc.ReflectUtil;
import java.io.ObjectStreamClass;
import java.io.InputStream;
import com.sun.jmx.remote.internal.ClientListenerInfo;
import java.rmi.ServerException;
import javax.management.MBeanServerDelegate;
import javax.management.NotificationFilterSupport;
import java.io.NotSerializableException;
import java.rmi.MarshalException;
import java.rmi.UnmarshalException;
import javax.management.remote.NotificationResult;
import com.sun.jmx.remote.internal.ClientNotifForwarder;
import javax.management.IntrospectionException;
import javax.management.MBeanInfo;
import javax.management.InvalidAttributeValueException;
import javax.management.AttributeNotFoundException;
import java.util.Set;
import javax.management.QueryExp;
import javax.management.NotCompliantMBeanException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.ReflectionException;
import javax.management.ObjectInstance;
import com.sun.jmx.remote.internal.ProxyRef;
import java.util.stream.Collectors;
import javax.management.Attribute;
import javax.management.AttributeList;
import java.util.Arrays;
import java.security.PrivilegedAction;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.lang.reflect.InvocationTargetException;
import java.io.ByteArrayInputStream;
import javax.naming.InitialContext;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.server.RemoteRef;
import java.lang.reflect.InvocationHandler;
import javax.rmi.ssl.SslRMIClientSocketFactory;
import sun.rmi.server.UnicastRef2;
import java.rmi.server.RemoteObject;
import java.rmi.server.RemoteObjectInvocationHandler;
import java.lang.reflect.Proxy;
import java.rmi.Remote;
import java.io.ObjectOutputStream;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.util.Properties;
import com.sun.jmx.remote.internal.IIOPHelper;
import javax.management.InstanceNotFoundException;
import com.sun.jmx.mbeanserver.Util;
import java.rmi.MarshalledObject;
import javax.management.ObjectName;
import java.rmi.NoSuchObjectException;
import javax.management.Notification;
import javax.management.ListenerNotFoundException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.naming.NamingException;
import javax.management.remote.JMXConnectionNotification;
import java.rmi.RemoteException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.io.IOException;
import java.util.Collections;
import com.sun.jmx.remote.util.EnvHelp;
import com.sun.jmx.remote.internal.ClientCommunicatorAdmin;
import javax.management.NotificationBroadcasterSupport;
import javax.management.MBeanServerConnection;
import java.lang.ref.WeakReference;
import javax.security.auth.Subject;
import java.util.WeakHashMap;
import java.util.Map;
import javax.management.remote.JMXServiceURL;
import java.lang.reflect.Constructor;
import com.sun.jmx.remote.util.ClassLogger;
import javax.management.remote.JMXAddressable;
import java.io.Serializable;
import javax.management.remote.JMXConnector;

public class RMIConnector implements JMXConnector, Serializable, JMXAddressable
{
    private static final ClassLogger logger;
    private static final long serialVersionUID = 817323035842634473L;
    private static final String rmiServerImplStubClassName;
    private static final Class<?> rmiServerImplStubClass;
    private static final String rmiConnectionImplStubClassName;
    private static final Class<?> rmiConnectionImplStubClass;
    private static final String pRefClassName = "com.sun.jmx.remote.internal.PRef";
    private static final Constructor<?> proxyRefConstructor;
    private static final String iiopConnectionStubClassName = "org.omg.stub.javax.management.remote.rmi._RMIConnection_Stub";
    private static final String proxyStubClassName = "com.sun.jmx.remote.protocol.iiop.ProxyStub";
    private static final String ProxyInputStreamClassName = "com.sun.jmx.remote.protocol.iiop.ProxyInputStream";
    private static final String pInputStreamClassName = "com.sun.jmx.remote.protocol.iiop.PInputStream";
    private static final Class<?> proxyStubClass;
    private static final byte[] base64ToInt;
    private final RMIServer rmiServer;
    private final JMXServiceURL jmxServiceURL;
    private transient Map<String, Object> env;
    private transient ClassLoader defaultClassLoader;
    private transient RMIConnection connection;
    private transient String connectionId;
    private transient long clientNotifSeqNo;
    private transient WeakHashMap<Subject, WeakReference<MBeanServerConnection>> rmbscMap;
    private transient WeakReference<MBeanServerConnection> nullSubjectConnRef;
    private transient RMIConnector.RMINotifClient rmiNotifClient;
    private transient long clientNotifCounter;
    private transient boolean connected;
    private transient boolean terminated;
    private transient Exception closeException;
    private transient NotificationBroadcasterSupport connectionBroadcaster;
    private transient ClientCommunicatorAdmin communicatorAdmin;
    private static volatile WeakReference<Object> orb;
    
    private RMIConnector(RMIServer rmiServer, JMXServiceURL jmxServiceURL, Map<String, ?> map) {
        this.clientNotifSeqNo = 0L;
        this.nullSubjectConnRef = null;
        this.clientNotifCounter = 0L;
        if (rmiServer == null && jmxServiceURL == null) {
            throw new IllegalArgumentException("rmiServer and jmxServiceURL both null");
        }
        this.initTransients();
        this.rmiServer = rmiServer;
        this.jmxServiceURL = jmxServiceURL;
        if (map == null) {
            this.env = Collections.emptyMap();
        }
        else {
            EnvHelp.checkAttributes(map);
            this.env = Collections.unmodifiableMap(map);
        }
        return;
    }
    
    public RMIConnector(JMXServiceURL jmxServiceURL, Map<String, ?> map) {
        this(null, jmxServiceURL, map);
    }
    
    public RMIConnector(RMIServer rmiServer, Map<String, ?> map) {
        this(rmiServer, null, map);
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(this.getClass().getName());
        sb.append(":");
        if (this.rmiServer != null) {
            sb.append(" rmiServer=").append(this.rmiServer.toString());
        }
        if (this.jmxServiceURL != null) {
            if (this.rmiServer != null) {
                sb.append(",");
            }
            sb.append(" jmxServiceURL=").append(this.jmxServiceURL.toString());
        }
        return sb.toString();
    }
    
    @Override
    public JMXServiceURL getAddress() {
        return this.jmxServiceURL;
    }
    
    @Override
    public void connect() throws IOException {
        this.connect(null);
    }
    
    @Override
    public synchronized void connect(Map<String, ?> attributes) throws IOException {
        int traceOn = RMIConnector.logger.traceOn();
        String str = traceOn ? ("[" + this.toString() + "]") : null;
        if (this.terminated) {
            RMIConnector.logger.trace("connect", str + " already closed.");
            throw new IOException("Connector closed");
        }
        if (this.connected) {
            RMIConnector.logger.trace("connect", str + " already connected.");
            return;
        }
        try {
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " connecting...");
            }
            HashMap env = new HashMap((this.env == null) ? Collections.emptyMap() : this.env);
            if (attributes != null) {
                EnvHelp.checkAttributes(attributes);
                env.putAll(attributes);
            }
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " finding stub...");
            }
            RMIServer rmiServer = (this.rmiServer != null) ? this.rmiServer : this.findRMIServer(this.jmxServiceURL, env);
            int computeBooleanFromString = EnvHelp.computeBooleanFromString((String)env.get("jmx.remote.x.check.stub"));
            if (!(computeBooleanFromString == 0)) {
                checkStub(rmiServer, RMIConnector.rmiServerImplStubClass);
            }
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " connecting stub...");
            }
            RMIServer connectStub = connectStub(rmiServer, env);
            str = (traceOn ? ("[" + this.toString() + "]") : null);
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " getting connection...");
            }
            Object value = env.get("jmx.remote.credentials");
            try {
                this.connection = getConnection(connectStub, value, (boolean)computeBooleanFromString);
            }
            catch (RemoteException cause) {
                if (this.jmxServiceURL != null) {
                    String protocol = this.jmxServiceURL.getProtocol();
                    String urlPath = this.jmxServiceURL.getURLPath();
                    if ("rmi".equals(protocol) && urlPath.startsWith("/jndi/iiop:")) {
                        MalformedURLException o = new MalformedURLException("Protocol is rmi but JNDI scheme is iiop: " + this.jmxServiceURL);
                        o.initCause(cause);
                        throw o;
                    }
                }
                throw cause;
            }
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " getting class loader...");
            }
            env.put("jmx.remote.default.class.loader", this.defaultClassLoader = EnvHelp.resolveClientClassLoader(env));
            this.rmiNotifClient = new RMIConnector.RMINotifClient(this.defaultClassLoader, env);
            this.env = env;
            this.communicatorAdmin = new RMIConnector.RMIClientCommunicatorAdmin(EnvHelp.getConnectionCheckPeriod(env));
            this.connected = true;
            this.connectionId = this.getConnectionId();
            this.sendNotification(new JMXConnectionNotification("jmx.remote.connection.opened", this, this.connectionId, this.clientNotifSeqNo++, "Successful connection", null));
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " done...");
            }
        }
        catch (IOException obj) {
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " failed to connect: " + obj);
            }
            throw obj;
        }
        catch (RuntimeException obj2) {
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " failed to connect: " + obj2);
            }
            throw obj2;
        }
        catch (NamingException ex) {
            String string = "Failed to retrieve RMIServer stub: " + ex;
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("connect", str + " " + string);
            }
            throw (IOException)EnvHelp.initCause(new IOException(string), ex);
        }
        return;
    }
    
    @Override
    public synchronized String getConnectionId() throws IOException {
        if (this.terminated || !this.connected) {
            if (RMIConnector.logger.traceOn()) {
                RMIConnector.logger.trace("getConnectionId", "[" + this.toString() + "] not connected.");
            }
            throw new IOException("Not connected");
        }
        else {
            return this.connection.getConnectionId();
        }
    }
    
    @Override
    public synchronized MBeanServerConnection getMBeanServerConnection() throws IOException {
        return this.getMBeanServerConnection(null);
    }
    
    @Override
    public synchronized MBeanServerConnection getMBeanServerConnection(Subject subject) throws IOException {
        if (this.terminated) {
            if (RMIConnector.logger.traceOn()) {
                RMIConnector.logger.trace("getMBeanServerConnection", "[" + this.toString() + "] already closed.");
            }
            throw new IOException("Connection closed");
        }
        else if (!this.connected) {
            if (RMIConnector.logger.traceOn()) {
                RMIConnector.logger.trace("getMBeanServerConnection", "[" + this.toString() + "] is not connected.");
            }
            throw new IOException("Not connected");
        }
        else {
            return this.getConnectionWithSubject(subject);
        }
    }
    
    @Override
    public void addConnectionNotificationListener(NotificationListener listener, NotificationFilter filter, Object handback) {
        if (listener == null) {
            throw new NullPointerException("listener");
        }
        this.connectionBroadcaster.addNotificationListener(listener, filter, handback);
        return;
    }
    
    @Override
    public void removeConnectionNotificationListener(NotificationListener listener) throws ListenerNotFoundException {
        if (listener == null) {
            throw new NullPointerException("listener");
        }
        this.connectionBroadcaster.removeNotificationListener(listener);
        return;
    }
    
    @Override
    public void removeConnectionNotificationListener(NotificationListener listener, NotificationFilter filter, Object handback) throws ListenerNotFoundException {
        if (listener == null) {
            throw new NullPointerException("listener");
        }
        this.connectionBroadcaster.removeNotificationListener(listener, filter, handback);
        return;
    }
    
    private void sendNotification(Notification notification) {
        this.connectionBroadcaster.sendNotification(notification);
    }
    
    @Override
    public synchronized void close() throws IOException {
        this.close(false);
    }
    
    private synchronized void close(boolean b) throws IOException {
        int traceOn = RMIConnector.logger.traceOn();
        int debugOn = RMIConnector.logger.debugOn();
        String str = traceOn ? ("[" + this.toString() + "]") : null;
        if (!b) {
            if (this.terminated) {
                if (this.closeException == null) {
                    if (!(traceOn == 0)) {
                        RMIConnector.logger.trace("close", str + " already closed.");
                    }
                    return;
                }
            }
            else {
                this.terminated = true;
            }
        }
        if (this.closeException != null && traceOn && traceOn) {
            RMIConnector.logger.trace("close", str + " had failed: " + this.closeException);
            RMIConnector.logger.trace("close", str + " attempting to close again.");
        }
        String connectionId = null;
        if (this.connected) {
            connectionId = this.connectionId;
        }
        this.closeException = null;
        if (!(traceOn == 0)) {
            RMIConnector.logger.trace("close", str + " closing.");
        }
        if (this.communicatorAdmin != null) {
            this.communicatorAdmin.terminate();
        }
        if (this.rmiNotifClient != null) {
            try {
                this.rmiNotifClient.terminate();
                if (!(traceOn == 0)) {
                    RMIConnector.logger.trace("close", str + " RMI Notification client terminated.");
                }
            }
            catch (RuntimeException t) {
                this.closeException = t;
                if (!(traceOn == 0)) {
                    RMIConnector.logger.trace("close", str + " Failed to terminate RMI Notification client: " + t);
                }
                if (!(debugOn == 0)) {
                    RMIConnector.logger.debug("close", t);
                }
            }
        }
        if (this.connection != null) {
            try {
                this.connection.close();
                if (!(traceOn == 0)) {
                    RMIConnector.logger.trace("close", str + " closed.");
                }
            }
            catch (NoSuchObjectException ex) {}
            catch (IOException t2) {
                this.closeException = t2;
                if (!(traceOn == 0)) {
                    RMIConnector.logger.trace("close", str + " Failed to close RMIServer: " + t2);
                }
                if (!(debugOn == 0)) {
                    RMIConnector.logger.debug("close", t2);
                }
            }
        }
        this.rmbscMap.clear();
        if (connectionId != null) {
            this.sendNotification(new JMXConnectionNotification("jmx.remote.connection.closed", this, connectionId, this.clientNotifSeqNo++, "Client has been closed", null));
        }
        if (this.closeException != null) {
            if (!(traceOn == 0)) {
                RMIConnector.logger.trace("close", str + " failed to close: " + this.closeException);
            }
            if ((this.closeException instanceof IOException)) {
                throw (IOException)this.closeException;
            }
            if ((this.closeException instanceof RuntimeException)) {
                throw (RuntimeException)this.closeException;
            }
            throw (IOException)EnvHelp.initCause(new IOException("Failed to close: " + this.closeException), this.closeException);
        }
        else {
            return;
        }
    }
    
    private Integer addListenerWithSubject(ObjectName objectName, MarshalledObject<NotificationFilter> marshalledObject, Subject subject, boolean b) throws InstanceNotFoundException, IOException {
        int debugOn = RMIConnector.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnector.logger.debug("addListenerWithSubject", "(ObjectName,MarshalledObject,Subject)");
        }
        Integer[] addListenersWithSubjects = this.addListenersWithSubjects(new ObjectName[] { objectName }, (MarshalledObject[])Util.cast(new MarshalledObject[] { marshalledObject }), new Subject[] { subject }, b);
        if (!(debugOn == 0)) {
            RMIConnector.logger.debug("addListenerWithSubject", "listenerID=" + addListenersWithSubjects[0]);
        }
        return addListenersWithSubjects[0];
    }
    
    private Integer[] addListenersWithSubjects(ObjectName[] array, MarshalledObject<NotificationFilter>[] array2, Subject[] array3, boolean b) throws InstanceNotFoundException, IOException {
        int debugOn = RMIConnector.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnector.logger.debug("addListenersWithSubjects", "(ObjectName[],MarshalledObject[],Subject[])");
        }
        ClassLoader pushDefaultClassLoader = this.pushDefaultClassLoader();
        Integer[] array4 = null;
        try {
            array4 = this.connection.addNotificationListeners(array, array2, array3);
        }
        catch (NoSuchObjectException ioe) {
            if (b) {
                this.communicatorAdmin.gotIOException(ioe);
                array4 = this.connection.addNotificationListeners(array, array2, array3);
                this.popDefaultClassLoader(pushDefaultClassLoader);
            }
            else {
                throw ioe;
            }
        }
        catch (IOException ioe2) {
            this.communicatorAdmin.gotIOException(ioe2);
        }
        finally {
            this.popDefaultClassLoader(pushDefaultClassLoader);
        }
        if (!(debugOn == 0)) {
            RMIConnector.logger.debug("addListenersWithSubjects", "registered " + ((array4 == null) ? 0 : array4.length) + " listener(s)");
            return array4;
        }
        return array4;
    }
    
    static RMIServer connectStub(RMIServer rmiServer, Map<String, ?> map) throws IOException {
        if (IIOPHelper.isStub(rmiServer)) {
            try {
                IIOPHelper.getOrb(rmiServer);
            }
            catch (UnsupportedOperationException ex) {
                IIOPHelper.connect(rmiServer, resolveOrb(map));
            }
        }
        return rmiServer;
    }
    
    static Object resolveOrb(Map<String, ?> map) throws IOException {
        if (map != null) {
            Object value = map.get("java.naming.corba.orb");
            if (value != null && !IIOPHelper.isOrb(value)) {
                throw new IllegalArgumentException("java.naming.corba.orb must be an instance of org.omg.CORBA.ORB.");
            }
            if (value != null) {
                return value;
            }
        }
        Object o = (RMIConnector.orb == null) ? null : RMIConnector.orb.get();
        if (o != null) {
            return o;
        }
        Object orb = IIOPHelper.createOrb((String[])null, (Properties)null);
        RMIConnector.orb = new WeakReference(orb);
        return orb;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.rmiServer == null && this.jmxServiceURL == null) {
            throw new InvalidObjectException("rmiServer and jmxServiceURL both null");
        }
        this.initTransients();
        return;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        if (this.rmiServer == null && this.jmxServiceURL == null) {
            throw new InvalidObjectException("rmiServer and jmxServiceURL both null.");
        }
        connectStub(this.rmiServer, this.env);
        objectOutputStream.defaultWriteObject();
        return;
    }
    
    private void initTransients() {
        this.rmbscMap = new WeakHashMap();
        this.connected = false;
        this.terminated = false;
        this.connectionBroadcaster = new NotificationBroadcasterSupport();
    }
    
    private static void checkStub(Remote proxy, Class<?> clazz) {
        if (proxy.getClass() != clazz) {
            if (!Proxy.isProxyClass(proxy.getClass())) {
                throw new SecurityException("Expecting a " + clazz.getName() + " stub!");
            }
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(proxy);
            if (invocationHandler.getClass() != RemoteObjectInvocationHandler.class) {
                throw new SecurityException("Expecting a dynamic proxy instance with a " + RemoteObjectInvocationHandler.class.getName() + " invocation handler!");
            }
            proxy = (Remote)invocationHandler;
        }
        RemoteRef ref = ((RemoteObject)proxy).getRef();
        if (ref.getClass() != UnicastRef2.class) {
            throw new SecurityException("Expecting a " + UnicastRef2.class.getName() + " remote reference in stub!");
        }
        RMIClientSocketFactory clientSocketFactory = ((UnicastRef2)ref).getLiveRef().getClientSocketFactory();
        if (clientSocketFactory == null || clientSocketFactory.getClass() != SslRMIClientSocketFactory.class) {
            throw new SecurityException("Expecting a " + SslRMIClientSocketFactory.class.getName() + " RMI client socket factory in stub!");
        }
        return;
    }
    
    private RMIServer findRMIServer(JMXServiceURL jmxServiceURL, Map<String, Object> map) throws NamingException, IOException {
        int iiopURL = RMIConnectorServer.isIiopURL(jmxServiceURL, true);
        if (!(iiopURL == 0)) {
            map.put("java.naming.corba.orb", resolveOrb(map));
        }
        String urlPath = jmxServiceURL.getURLPath();
        int endIndex = urlPath.indexOf(59);
        if (endIndex < 0) {
            endIndex = urlPath.length();
        }
        if (urlPath.startsWith("/jndi/")) {
            return this.findRMIServerJNDI(urlPath.substring(6, endIndex), map, (boolean)iiopURL);
        }
        if (urlPath.startsWith("/stub/")) {
            return this.findRMIServerJRMP(urlPath.substring(6, endIndex), map, (boolean)iiopURL);
        }
        if (urlPath.startsWith("/ior/")) {
            if (!IIOPHelper.isAvailable()) {
                throw new IOException("iiop protocol not available");
            }
            return this.findRMIServerIIOP(urlPath.substring(5, endIndex), map, (boolean)iiopURL);
        }
        else {
            throw new MalformedURLException("URL path must begin with /jndi/ or /stub/ or /ior/: " + urlPath);
        }
    }
    
    private RMIServer findRMIServerJNDI(String name, Map<String, ?> map, boolean b) throws NamingException {
        InitialContext initialContext = new InitialContext(EnvHelp.mapToHashtable(map));
        Object lookup = initialContext.lookup(name);
        initialContext.close();
        if (b) {
            return narrowIIOPServer(lookup);
        }
        return narrowJRMPServer(lookup);
    }
    
    private static RMIServer narrowJRMPServer(Object o) {
        return (RMIServer)o;
    }
    
    private static RMIServer narrowIIOPServer(Object obj) {
        try {
            return (RMIServer)IIOPHelper.narrow(obj, RMIServer.class);
        }
        catch (ClassCastException ex) {
            if (RMIConnector.logger.traceOn()) {
                RMIConnector.logger.trace("narrowIIOPServer", "Failed to narrow objref=" + obj + ": " + ex);
            }
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("narrowIIOPServer", ex);
            }
            return null;
        }
    }
    
    private RMIServer findRMIServerIIOP(String s, Map<String, ?> map, boolean b) {
        return (RMIServer)IIOPHelper.narrow(IIOPHelper.stringToObject(map.get("java.naming.corba.orb"), s), RMIServer.class);
    }
    
    private RMIServer findRMIServerJRMP(String s, Map<String, ?> env, boolean b) throws IOException {
        byte[] base64ToByteArray;
        try {
            base64ToByteArray = base64ToByteArray(s);
        }
        catch (IllegalArgumentException ex) {
            throw new MalformedURLException("Bad BASE64 encoding: " + ex.getMessage());
        }
        ByteArrayInputStream in = new ByteArrayInputStream(base64ToByteArray);
        ClassLoader resolveClientClassLoader = EnvHelp.resolveClientClassLoader(env);
        ObjectInputStream objectInputStream = (resolveClientClassLoader == null) ? new ObjectInputStream(in) : new RMIConnector.ObjectInputStreamWithLoader(in, resolveClientClassLoader);
        Object object;
        try {
            object = objectInputStream.readObject();
        }
        catch (ClassNotFoundException obj) {
            throw new MalformedURLException("Class not found: " + obj);
        }
        return (RMIServer)object;
    }
    
    private MBeanServerConnection getConnectionWithSubject(Subject subject) {
        MBeanServerConnection mBeanServerConnection;
        if (subject == null) {
            if (this.nullSubjectConnRef == null || (mBeanServerConnection = (MBeanServerConnection)this.nullSubjectConnRef.get()) == null) {
                mBeanServerConnection = new RMIConnector.RemoteMBeanServerConnection(null);
                this.nullSubjectConnRef = new WeakReference(mBeanServerConnection);
            }
        }
        else {
            WeakReference<Object> weakReference = (WeakReference)this.rmbscMap.get(subject);
            if (weakReference == null || (mBeanServerConnection = (MBeanServerConnection)weakReference.get()) == null) {
                mBeanServerConnection = new RMIConnector.RemoteMBeanServerConnection(subject);
                this.rmbscMap.put(subject, new WeakReference(mBeanServerConnection));
            }
        }
        return mBeanServerConnection;
    }
    
    private static RMIConnection shadowJrmpStub(RemoteObject remoteObject) throws InstantiationException, IllegalAccessException, InvocationTargetException, ClassNotFoundException, NoSuchMethodException {
        return (RMIConnection)RMIConnector.rmiConnectionImplStubClass.getConstructor(new Class[] { RemoteRef.class }).newInstance(new Object[] { (RemoteRef)RMIConnector.proxyRefConstructor.newInstance(new Object[] { remoteObject.getRef() }) });
    }
    
    private static RMIConnection shadowIiopStub(Object o) throws InstantiationException, IllegalAccessException {
        Object doPrivileged;
        try {
            doPrivileged = AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
                @Override
                public Object run() throws Exception {
                    return RMIConnector.proxyStubClass.newInstance();
                }
            });
        }
        catch (PrivilegedActionException ex) {
            throw new InternalError();
        }
        IIOPHelper.setDelegate(doPrivileged, IIOPHelper.getDelegate(o));
        return (RMIConnection)doPrivileged;
    }
    
    private static RMIConnection getConnection(RMIServer rmiServer, Object o, boolean b) throws IOException {
        RMIConnection client = rmiServer.newClient(o);
        if (b) {
            checkStub(client, RMIConnector.rmiConnectionImplStubClass);
        }
        try {
            if (client.getClass() == RMIConnector.rmiConnectionImplStubClass) {
                return shadowJrmpStub((RemoteObject)client);
            }
            if (client.getClass().getName().equals("org.omg.stub.javax.management.remote.rmi._RMIConnection_Stub")) {
                return shadowIiopStub(client);
            }
            RMIConnector.logger.trace("getConnection", "Did not wrap " + client.getClass() + " to foil stack search for classes: class loading semantics may be incorrect");
        }
        catch (Exception ex) {
            RMIConnector.logger.error("getConnection", "Could not wrap " + client.getClass() + " to foil stack search for classes: class loading semantics may be incorrect: " + ex);
            RMIConnector.logger.debug("getConnection", ex);
        }
        return client;
    }
    
    private static byte[] base64ToByteArray(String s) {
        int length = s.length();
        int n = length / 4;
        if (4 * n != length) {
            throw new IllegalArgumentException("String length must be a multiple of four.");
        }
        int n2 = 0;
        if (length != 0) {
            if (s.charAt(length - 1) == '=') {
                ++n2;
                --n;
            }
            if (s.charAt(length - 2) == '=') {
                ++n2;
            }
        }
        byte[] array = new byte[3 * n - n2];
        int n3 = 0;
        int n4 = 0;
        for (int i = 0; i < n; ++i) {
            int base64toInt = base64toInt(s.charAt(n3++));
            int base64toInt2 = base64toInt(s.charAt(n3++));
            int base64toInt3 = base64toInt(s.charAt(n3++));
            int base64toInt4 = base64toInt(s.charAt(n3++));
            array[n4++] = (byte)(base64toInt << 2 | base64toInt2 >> 4);
            array[n4++] = (byte)(base64toInt2 << 4 | base64toInt3 >> 2);
            array[n4++] = (byte)(base64toInt3 << 6 | base64toInt4);
        }
        if (n2 != 0) {
            int base64toInt5 = base64toInt(s.charAt(n3++));
            int base64toInt6 = base64toInt(s.charAt(n3++));
            array[n4++] = (byte)(base64toInt5 << 2 | base64toInt6 >> 4);
            if (n2 == 1) {
                array[n4++] = (byte)(base64toInt6 << 4 | base64toInt(s.charAt(n3++)) >> 2);
            }
        }
        return array;
    }
    
    private static int base64toInt(char c) {
        int n;
        if (c >= RMIConnector.base64ToInt.length) {
            n = -1;
        }
        else {
            n = RMIConnector.base64ToInt[c];
        }
        if (n < 0) {
            throw new IllegalArgumentException("Illegal character " + c);
        }
        return n;
    }
    
    private ClassLoader pushDefaultClassLoader() {
        final Thread currentThread = Thread.currentThread();
        ClassLoader contextClassLoader = currentThread.getContextClassLoader();
        if (this.defaultClassLoader != null) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    currentThread.setContextClassLoader(RMIConnector.this.defaultClassLoader);
                    return null;
                }
            });
            return contextClassLoader;
        }
        return contextClassLoader;
    }
    
    private void popDefaultClassLoader(final ClassLoader classLoader) {
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                Thread.currentThread().setContextClassLoader(classLoader);
                return null;
            }
        });
    }
    
    private static String objects(Object[] a) {
        if (a == null) {
            return "null";
        }
        return Arrays.asList(a).toString();
    }
    
    private static String strings(String[] array) {
        return objects(array);
    }
    
    static String getAttributesNames(AttributeList list) {
        return (list != null) ? ((String)list.asList().stream().map(Attribute::getName).collect(Collectors.joining(", ", "[", "]"))) : "[]";
    }
    
    static ClassLoader access$100(RMIConnector rmiConnector) {
        return rmiConnector.pushDefaultClassLoader();
    }
    
    static void access$400(RMIConnector rmiConnector, ClassLoader classLoader) {
        rmiConnector.popDefaultClassLoader(classLoader);
    }
    
    static String access$500(String[] array) {
        return strings(array);
    }
    
    static Integer access$600(RMIConnector rmiConnector, ObjectName objectName, MarshalledObject marshalledObject, Subject subject, boolean b) throws InstanceNotFoundException, IOException {
        return rmiConnector.addListenerWithSubject(objectName, marshalledObject, subject, b);
    }
    
    static String access$800(Object[] array) {
        return objects(array);
    }
    
    static void access$1200(RMIConnector rmiConnector, Notification notification) {
        rmiConnector.sendNotification(notification);
    }
    
    static void access$1300(RMIConnector rmiConnector, boolean b) throws IOException {
        rmiConnector.close(b);
    }
    
    static Integer[] access$1500(RMIConnector rmiConnector, ObjectName[] array, MarshalledObject[] array2, Subject[] array3, boolean b) throws InstanceNotFoundException, IOException {
        return rmiConnector.addListenersWithSubjects(array, array2, array3, b);
    }
    
    static RMIServer access$1900(RMIConnector rmiConnector, JMXServiceURL jmxServiceURL, Map map) throws NamingException, IOException {
        return rmiConnector.findRMIServer(jmxServiceURL, map);
    }
    
    static {
        logger = new ClassLogger("javax.management.remote.rmi", "RMIConnector");
        rmiServerImplStubClassName = RMIServer.class.getName() + "Impl_Stub";
        rmiConnectionImplStubClassName = RMIConnection.class.getName() + "Impl_Stub";
        PrivilegedExceptionAction<Constructor<?>> action = new PrivilegedExceptionAction<Constructor<?>>() {
            
            @Override
            public Constructor<?> run() throws Exception {
                Class<Object> clazz = RMIConnector.class;
                return new NoCallStackClassLoader("com.sun.jmx.remote.internal.PRef", pRefByteCode, new String[] { ProxyRef.class.getName() }, clazz.getClassLoader(), clazz.getProtectionDomain()).loadClass("com.sun.jmx.remote.internal.PRef").getConstructor(new Class[] { RemoteRef.class });
            }
        };
        Class<Object> forName;
        try {
            forName = Class.forName(RMIConnector.rmiServerImplStubClassName);
        }
        catch (Exception ex) {
            RMIConnector.logger.error("<clinit>", "Failed to instantiate " + RMIConnector.rmiServerImplStubClassName + ": " + ex);
            RMIConnector.logger.debug("<clinit>", ex);
            forName = null;
        }
        rmiServerImplStubClass = forName;
        Class<Object> forName2;
        Constructor<Object> proxyRefConstructor2;
        try {
            forName2 = Class.forName(RMIConnector.rmiConnectionImplStubClassName);
            proxyRefConstructor2 = (Constructor)AccessController.doPrivileged(action);
        }
        catch (Exception ex2) {
            RMIConnector.logger.error("<clinit>", "Failed to initialize proxy reference constructor for " + RMIConnector.rmiConnectionImplStubClassName + ": " + ex2);
            RMIConnector.logger.debug("<clinit>", ex2);
            forName2 = null;
            proxyRefConstructor2 = null;
        }
        rmiConnectionImplStubClass = forName2;
        proxyRefConstructor = proxyRefConstructor2;
        byte[] stringToBytes = NoCallStackClassLoader.stringToBytes("Êþº¾\u0000\u0000\u00003\u0000+\n\u0000\f\u0000\u0018\u0007\u0000\u0019\n\u0000\f\u0000\u001a\n\u0000\u0002\u0000\u001b\u0007\u0000\u001c\n\u0000\u0005\u0000\u001d\n\u0000\u0005\u0000\u001e\n\u0000\u0005\u0000\u001f\n\u0000\u0002\u0000 \n\u0000\f\u0000!\u0007\u0000\"\u0007\u0000#\u0001\u0000\u0006<init>\u0001\u0000\u0003()V\u0001\u0000\u0004Code\u0001\u0000\u0007_invoke\u0001\u0000K(Lorg/omg/CORBA/portable/OutputStream;)Lorg/omg/CORBA/portable/InputStream;\u0001\u0000\rStackMapTable\u0007\u0000\u001c\u0001\u0000\nExceptions\u0007\u0000$\u0001\u0000\r_releaseReply\u0001\u0000'(Lorg/omg/CORBA/portable/InputStream;)V\f\u0000\r\u0000\u000e\u0001\u0000-com/sun/jmx/remote/protocol/iiop/PInputStream\f\u0000\u0010\u0000\u0011\f\u0000\r\u0000\u0017\u0001\u0000+org/omg/CORBA/portable/ApplicationException\f\u0000%\u0000&\f\u0000'\u0000(\f\u0000\r\u0000)\f\u0000*\u0000&\f\u0000\u0016\u0000\u0017\u0001\u0000*com/sun/jmx/remote/protocol/iiop/ProxyStub\u0001\u0000<org/omg/stub/javax/management/remote/rmi/_RMIConnection_Stub\u0001\u0000)org/omg/CORBA/portable/RemarshalException\u0001\u0000\u000egetInputStream\u0001\u0000&()Lorg/omg/CORBA/portable/InputStream;\u0001\u0000\u0005getId\u0001\u0000\u0014()Ljava/lang/String;\u0001\u00009(Ljava/lang/String;Lorg/omg/CORBA/portable/InputStream;)V\u0001\u0000\u0015getProxiedInputStream\u0000!\u0000\u000b\u0000\f\u0000\u0000\u0000\u0000\u0000\u0003\u0000\u0001\u0000\r\u0000\u000e\u0000\u0001\u0000\u000f\u0000\u0000\u0000\u0011\u0000\u0001\u0000\u0001\u0000\u0000\u0000\u0005*·\u0000\u0001±\u0000\u0000\u0000\u0000\u0000\u0001\u0000\u0010\u0000\u0011\u0000\u0002\u0000\u000f\u0000\u0000\u0000G\u0000\u0004\u0000\u0004\u0000\u0000\u0000'»\u0000\u0002Y*+·\u0000\u0003·\u0000\u0004°M»\u0000\u0002Y,¶\u0000\u0006·\u0000\u0004N»\u0000\u0005Y,¶\u0000\u0007-·\u0000\b¿\u0000\u0001\u0000\u0000\u0000\f\u0000\r\u0000\u0005\u0000\u0001\u0000\u0012\u0000\u0000\u0000\u0006\u0000\u0001M\u0007\u0000\u0013\u0000\u0014\u0000\u0000\u0000\u0006\u0000\u0002\u0000\u0005\u0000\u0015\u0000\u0001\u0000\u0016\u0000\u0017\u0000\u0001\u0000\u000f\u0000\u0000\u0000'\u0000\u0002\u0000\u0002\u0000\u0000\u0000\u0012+Æ\u0000\u000b+À\u0000\u0002¶\u0000\tL*+·\u0000\n±\u0000\u0000\u0000\u0001\u0000\u0012\u0000\u0000\u0000\u0003\u0000\u0001\f\u0000\u0000");
        byte[] stringToBytes2 = NoCallStackClassLoader.stringToBytes("Êþº¾\u0000\u0000\u00003\u0000\u001e\n\u0000\u0007\u0000\u000f\t\u0000\u0006\u0000\u0010\n\u0000\u0011\u0000\u0012\n\u0000\u0006\u0000\u0013\n\u0000\u0014\u0000\u0015\u0007\u0000\u0016\u0007\u0000\u0017\u0001\u0000\u0006<init>\u0001\u0000'(Lorg/omg/CORBA/portable/InputStream;)V\u0001\u0000\u0004Code\u0001\u0000\bread_any\u0001\u0000\u0015()Lorg/omg/CORBA/Any;\u0001\u0000\nread_value\u0001\u0000)(Ljava/lang/Class;)Ljava/io/Serializable;\f\u0000\b\u0000\t\f\u0000\u0018\u0000\u0019\u0007\u0000\u001a\f\u0000\u000b\u0000\f\f\u0000\u001b\u0000\u001c\u0007\u0000\u001d\f\u0000\r\u0000\u000e\u0001\u0000-com/sun/jmx/remote/protocol/iiop/PInputStream\u0001\u00001com/sun/jmx/remote/protocol/iiop/ProxyInputStream\u0001\u0000\u0002in\u0001\u0000$Lorg/omg/CORBA/portable/InputStream;\u0001\u0000\"org/omg/CORBA/portable/InputStream\u0001\u0000\u0006narrow\u0001\u0000*()Lorg/omg/CORBA_2_3/portable/InputStream;\u0001\u0000&org/omg/CORBA_2_3/portable/InputStream\u0000!\u0000\u0006\u0000\u0007\u0000\u0000\u0000\u0000\u0000\u0003\u0000\u0001\u0000\b\u0000\t\u0000\u0001\u0000\n\u0000\u0000\u0000\u0012\u0000\u0002\u0000\u0002\u0000\u0000\u0000\u0006*+·\u0000\u0001±\u0000\u0000\u0000\u0000\u0000\u0001\u0000\u000b\u0000\f\u0000\u0001\u0000\n\u0000\u0000\u0000\u0014\u0000\u0001\u0000\u0001\u0000\u0000\u0000\b*´\u0000\u0002¶\u0000\u0003°\u0000\u0000\u0000\u0000\u0000\u0001\u0000\r\u0000\u000e\u0000\u0001\u0000\n\u0000\u0000\u0000\u0015\u0000\u0002\u0000\u0002\u0000\u0000\u0000\t*¶\u0000\u0004+¶\u0000\u0005°\u0000\u0000\u0000\u0000\u0000\u0000");
        final String[] array = { "com.sun.jmx.remote.protocol.iiop.ProxyStub", "com.sun.jmx.remote.protocol.iiop.PInputStream" };
        final byte[][] array2 = { stringToBytes, stringToBytes2 };
        final String[] array3 = { "org.omg.stub.javax.management.remote.rmi._RMIConnection_Stub", "com.sun.jmx.remote.protocol.iiop.ProxyInputStream" };
        if (IIOPHelper.isAvailable()) {
            PrivilegedExceptionAction<Class<?>> action2 = new PrivilegedExceptionAction<Class<?>>() {
                @Override
                public Class<?> run() throws Exception {
                    Class<Object> clazz = RMIConnector.class;
                    return new NoCallStackClassLoader(array, array2, array3, clazz.getClassLoader(), clazz.getProtectionDomain()).loadClass("com.sun.jmx.remote.protocol.iiop.ProxyStub");
                }
            };
            Class<Object> proxyStubClass2;
            try {
                proxyStubClass2 = (Class)AccessController.doPrivileged(action2);
            }
            catch (Exception ex3) {
                RMIConnector.logger.error("<clinit>", "Unexpected exception making shadow IIOP stub class: " + ex3);
                RMIConnector.logger.debug("<clinit>", ex3);
                proxyStubClass2 = null;
            }
            proxyStubClass = proxyStubClass2;
        }
        else {
            proxyStubClass = null;
        }
        base64ToInt = new byte[] { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1, -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51 };
        RMIConnector.orb = null;
    }
    
    private class RemoteMBeanServerConnection implements MBeanServerConnection
    {
        private Subject delegationSubject;
        
        public RemoteMBeanServerConnection(RMIConnector rmiConnector) {
            this(rmiConnector, null);
        }
        
        public RemoteMBeanServerConnection(Subject delegationSubject) {
            this.delegationSubject = delegationSubject;
        }
        
        @Override
        public ObjectInstance createMBean(String str, ObjectName obj) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("createMBean(String,ObjectName)", "className=" + str + ", name=" + obj);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.createMBean(str, obj, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.createMBean(str, obj, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public ObjectInstance createMBean(String str, ObjectName obj, ObjectName obj2) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("createMBean(String,ObjectName,ObjectName)", "className=" + str + ", name=" + obj + ", loaderName=" + obj2 + ")");
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.createMBean(str, obj, obj2, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.createMBean(str, obj, obj2, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public ObjectInstance createMBean(String str, ObjectName obj, Object[] obj2, String[] array) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("createMBean(String,ObjectName,Object[],String[])", "className=" + str + ", name=" + obj + ", signature=" + RMIConnector.access$500(array));
            }
            MarshalledObject<Object> marshalledObject = new MarshalledObject(obj2);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.createMBean(str, obj, marshalledObject, array, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.createMBean(str, obj, marshalledObject, array, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public ObjectInstance createMBean(String str, ObjectName obj, ObjectName obj2, Object[] obj3, String[] array) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("createMBean(String,ObjectName,ObjectName,Object[],String[])", "className=" + str + ", name=" + obj + ", loaderName=" + obj2 + ", signature=" + RMIConnector.access$500(array));
            }
            MarshalledObject<Object> marshalledObject = new MarshalledObject(obj3);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.createMBean(str, obj, obj2, marshalledObject, array, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.createMBean(str, obj, obj2, marshalledObject, array, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public void unregisterMBean(ObjectName obj) throws InstanceNotFoundException, MBeanRegistrationException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("unregisterMBean", "name=" + obj);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                RMIConnector.this.connection.unregisterMBean(obj, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.unregisterMBean(obj, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public ObjectInstance getObjectInstance(ObjectName obj) throws InstanceNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("getObjectInstance", "name=" + obj);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.getObjectInstance(obj, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.getObjectInstance(obj, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public Set<ObjectInstance> queryMBeans(ObjectName obj, QueryExp queryExp) throws IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("queryMBeans", "name=" + obj + ", query=" + queryExp);
            }
            MarshalledObject<QueryExp> marshalledObject = new MarshalledObject(queryExp);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.queryMBeans(obj, marshalledObject, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.queryMBeans(obj, marshalledObject, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public Set<ObjectName> queryNames(ObjectName obj, QueryExp queryExp) throws IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("queryNames", "name=" + obj + ", query=" + queryExp);
            }
            MarshalledObject<QueryExp> marshalledObject = new MarshalledObject(queryExp);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.queryNames(obj, marshalledObject, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.queryNames(obj, marshalledObject, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public boolean isRegistered(ObjectName obj) throws IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("isRegistered", "name=" + obj);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.isRegistered(obj, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.isRegistered(obj, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public Integer getMBeanCount() throws IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("getMBeanCount", "");
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.getMBeanCount(this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.getMBeanCount(this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public Object getAttribute(ObjectName obj, String str) throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("getAttribute", "name=" + obj + ", attribute=" + str);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.getAttribute(obj, str, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.getAttribute(obj, str, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public AttributeList getAttributes(ObjectName obj, String[] array) throws InstanceNotFoundException, ReflectionException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("getAttributes", "name=" + obj + ", attributes=" + RMIConnector.access$500(array));
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.getAttributes(obj, array, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.getAttributes(obj, array, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public void setAttribute(ObjectName obj, Attribute obj2) throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("setAttribute", "name=" + obj + ", attribute name=" + obj2.getName());
            }
            MarshalledObject<Attribute> marshalledObject = new MarshalledObject(obj2);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                RMIConnector.this.connection.setAttribute(obj, marshalledObject, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.setAttribute(obj, marshalledObject, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public AttributeList setAttributes(ObjectName obj, AttributeList obj2) throws InstanceNotFoundException, ReflectionException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("setAttributes", "name=" + obj + ", attribute names=" + RMIConnector.getAttributesNames(obj2));
            }
            MarshalledObject<AttributeList> marshalledObject = new MarshalledObject(obj2);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.setAttributes(obj, marshalledObject, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.setAttributes(obj, marshalledObject, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public Object invoke(ObjectName obj, String str, Object[] obj2, String[] array) throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("invoke", "name=" + obj + ", operationName=" + str + ", signature=" + RMIConnector.access$500(array));
            }
            MarshalledObject<Object> marshalledObject = new MarshalledObject(obj2);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.invoke(obj, str, marshalledObject, array, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.invoke(obj, str, marshalledObject, array, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public String getDefaultDomain() throws IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("getDefaultDomain", "");
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.getDefaultDomain(this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.getDefaultDomain(this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public String[] getDomains() throws IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("getDomains", "");
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.getDomains(this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.getDomains(this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public MBeanInfo getMBeanInfo(ObjectName obj) throws InstanceNotFoundException, IntrospectionException, ReflectionException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("getMBeanInfo", "name=" + obj);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.getMBeanInfo(obj, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.getMBeanInfo(obj, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public boolean isInstanceOf(ObjectName obj, String str) throws InstanceNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("isInstanceOf", "name=" + obj + ", className=" + str);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                return RMIConnector.this.connection.isInstanceOf(obj, str, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                return RMIConnector.this.connection.isInstanceOf(obj, str, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public void addNotificationListener(ObjectName obj, ObjectName obj2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("addNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "name=" + obj + ", listener=" + obj2 + ", filter=" + notificationFilter + ", handback=" + o);
            }
            MarshalledObject<NotificationFilter> marshalledObject = new MarshalledObject(notificationFilter);
            MarshalledObject<Object> marshalledObject2 = new MarshalledObject(o);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                RMIConnector.this.connection.addNotificationListener(obj, obj2, marshalledObject, marshalledObject2, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.addNotificationListener(obj, obj2, marshalledObject, marshalledObject2, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public void removeNotificationListener(ObjectName obj, ObjectName obj2) throws InstanceNotFoundException, ListenerNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("removeNotificationListener(ObjectName,ObjectName)", "name=" + obj + ", listener=" + obj2);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                RMIConnector.this.connection.removeNotificationListener(obj, obj2, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.removeNotificationListener(obj, obj2, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public void removeNotificationListener(ObjectName obj, ObjectName obj2, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("removeNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "name=" + obj + ", listener=" + obj2 + ", filter=" + notificationFilter + ", handback=" + o);
            }
            MarshalledObject<NotificationFilter> marshalledObject = new MarshalledObject(notificationFilter);
            MarshalledObject<Object> marshalledObject2 = new MarshalledObject(o);
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                RMIConnector.this.connection.removeNotificationListener(obj, obj2, marshalledObject, marshalledObject2, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.removeNotificationListener(obj, obj2, marshalledObject, marshalledObject2, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public void addNotificationListener(ObjectName objectName, NotificationListener notificationListener, NotificationFilter obj, Object o) throws InstanceNotFoundException, IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("addNotificationListener(ObjectName,NotificationListener,NotificationFilter,Object)", "name=" + objectName + ", listener=" + notificationListener + ", filter=" + obj + ", handback=" + o);
            }
            RMIConnector.this.rmiNotifClient.addNotificationListener(RMIConnector.access$600(RMIConnector.this, objectName, new MarshalledObject(obj), this.delegationSubject, true), objectName, notificationListener, obj, o, this.delegationSubject);
        }
        
        @Override
        public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener) throws InstanceNotFoundException, ListenerNotFoundException, IOException {
            int debugOn = RMIConnector.logger.debugOn();
            if (!(debugOn == 0)) {
                RMIConnector.logger.debug("removeNotificationListener(ObjectName,NotificationListener)", "name=" + objectName + ", listener=" + notificationListener);
            }
            Integer[] removeNotificationListener = RMIConnector.this.rmiNotifClient.removeNotificationListener(objectName, notificationListener);
            if (!(debugOn == 0)) {
                RMIConnector.logger.debug("removeNotificationListener", "listenerIDs=" + RMIConnector.access$800(removeNotificationListener));
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                RMIConnector.this.connection.removeNotificationListeners(objectName, removeNotificationListener, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.removeNotificationListeners(objectName, removeNotificationListener, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
        
        @Override
        public void removeNotificationListener(ObjectName objectName, NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws InstanceNotFoundException, ListenerNotFoundException, IOException {
            int debugOn = RMIConnector.logger.debugOn();
            if (!(debugOn == 0)) {
                RMIConnector.logger.debug("removeNotificationListener(ObjectName,NotificationListener,NotificationFilter,Object)", "name=" + objectName + ", listener=" + notificationListener + ", filter=" + notificationFilter + ", handback=" + o);
            }
            Integer removeNotificationListener = RMIConnector.this.rmiNotifClient.removeNotificationListener(objectName, notificationListener, notificationFilter, o);
            if (!(debugOn == 0)) {
                RMIConnector.logger.debug("removeNotificationListener", "listenerID=" + removeNotificationListener);
            }
            ClassLoader access$100 = RMIConnector.access$100(RMIConnector.this);
            try {
                RMIConnector.this.connection.removeNotificationListeners(objectName, new Integer[] { removeNotificationListener }, this.delegationSubject);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.removeNotificationListeners(objectName, new Integer[] { removeNotificationListener }, this.delegationSubject);
            }
            finally {
                RMIConnector.access$400(RMIConnector.this, access$100);
            }
        }
    }
    
    private class RMINotifClient extends ClientNotifForwarder
    {
        public RMINotifClient(ClassLoader defaultClassLoader, Map<String, ?> env) {
            super(defaultClassLoader, env);
        }
        
        @Override
        protected NotificationResult fetchNotifs(long n, int n2, long n3) throws IOException, ClassNotFoundException {
            int n4 = 0;
            try {
                return RMIConnector.this.connection.fetchNotifications(n, n2, n3);
            }
            catch (IOException ex) {
                this.rethrowDeserializationException(ex);
                try {
                    RMIConnector.this.communicatorAdmin.gotIOException(ex);
                }
                catch (IOException ex2) {
                    boolean n5 = false;
                    synchronized (this) {
                        if (RMIConnector.this.terminated) {
                            throw ex;
                        }
                        if (n4 != 0) {
                            n5 = true;
                        }
                    }
                    if (!(n5 == 0)) {
                        RMIConnector.access$1200(RMIConnector.this, new JMXConnectionNotification("jmx.remote.connection.failed", this, RMIConnector.this.connectionId, RMIConnector.this.clientNotifSeqNo++, "Failed to communicate with the server: " + ex.toString(), ex));
                        try {
                            RMIConnector.access$1300(RMIConnector.this, true);
                        }
                        catch (Exception ex3) {}
                        throw ex;
                    }
                    n4 = 1;
                }
                return RMIConnector.this.connection.fetchNotifications(n, n2, n3);
            }
        }
        
        private void rethrowDeserializationException(IOException ex) throws ClassNotFoundException, IOException {
            if ((ex instanceof UnmarshalException)) {
                throw ex;
            }
            if ((ex instanceof MarshalException)) {
                MarshalException ex2 = (MarshalException)ex;
                if ((ex2.detail instanceof NotSerializableException)) {
                    throw (NotSerializableException)ex2.detail;
                }
            }
            return;
        }
        
        @Override
        protected Integer addListenerForMBeanRemovedNotif() throws IOException, InstanceNotFoundException {
            NotificationFilterSupport obj = new NotificationFilterSupport();
            obj.enableType("JMX.mbean.unregistered");
            MarshalledObject<Object> marshalledObject = new MarshalledObject(obj);
            ObjectName[] array = { MBeanServerDelegate.DELEGATE_NAME };
            MarshalledObject[] array2 = (MarshalledObject[])Util.cast(new MarshalledObject[] { marshalledObject });
            Subject[] array3 = { null };
            Integer[] array4;
            try {
                array4 = RMIConnector.this.connection.addNotificationListeners(array, array2, array3);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                array4 = RMIConnector.this.connection.addNotificationListeners(array, array2, array3);
            }
            return array4[0];
        }
        
        @Override
        protected void removeListenerForMBeanRemovedNotif(Integer n) throws IOException, InstanceNotFoundException, ListenerNotFoundException {
            try {
                RMIConnector.this.connection.removeNotificationListeners(MBeanServerDelegate.DELEGATE_NAME, new Integer[] { n }, null);
            }
            catch (IOException ioe) {
                RMIConnector.this.communicatorAdmin.gotIOException(ioe);
                RMIConnector.this.connection.removeNotificationListeners(MBeanServerDelegate.DELEGATE_NAME, new Integer[] { n }, null);
            }
        }
        
        @Override
        protected void lostNotifs(String message, long l) {
            RMIConnector.access$1200(RMIConnector.this, new JMXConnectionNotification("jmx.remote.connection.notifs.lost", RMIConnector.this, RMIConnector.this.connectionId, RMIConnector.this.clientNotifCounter++, message, Long.valueOf(l)));
        }
    }
    
    private class RMIClientCommunicatorAdmin extends ClientCommunicatorAdmin
    {
        public RMIClientCommunicatorAdmin(long period) {
            super(period);
        }
        
        @Override
        public void gotIOException(IOException ex) throws IOException {
            if ((ex instanceof NoSuchObjectException)) {
                super.gotIOException(ex);
                return;
            }
            try {
                RMIConnector.this.connection.getDefaultDomain(null);
            }
            catch (IOException ex2) {
                boolean n = false;
                synchronized (this) {
                    if (!RMIConnector.this.terminated) {
                        RMIConnector.this.terminated = true;
                        n = true;
                    }
                }
                if (!(n == 0)) {
                    RMIConnector.access$1200(RMIConnector.this, new JMXConnectionNotification("jmx.remote.connection.failed", this, RMIConnector.this.connectionId, RMIConnector.this.clientNotifSeqNo++, "Failed to communicate with the server: " + ex.toString(), ex));
                    try {
                        RMIConnector.access$1300(RMIConnector.this, true);
                    }
                    catch (Exception ex3) {}
                }
            }
            if ((ex instanceof ServerException)) {
                Throwable detail = ((ServerException)ex).detail;
                if ((detail instanceof IOException)) {
                    throw (IOException)detail;
                }
                if ((detail instanceof RuntimeException)) {
                    throw (RuntimeException)detail;
                }
            }
            throw ex;
        }
        
        public void reconnectNotificationListeners(ClientListenerInfo[] array) throws IOException {
            int length = array.length;
            ClientListenerInfo[] array2 = new ClientListenerInfo[length];
            Subject[] array3 = new Subject[length];
            ObjectName[] array4 = new ObjectName[length];
            NotificationListener[] array5 = new NotificationListener[length];
            NotificationFilter[] array6 = new NotificationFilter[length];
            MarshalledObject[] array7 = (MarshalledObject[])Util.cast(new MarshalledObject[length]);
            Object[] array8 = new Object[length];
            for (int i = 0; i < length; ++i) {
                array3[i] = array[i].getDelegationSubject();
                array4[i] = array[i].getObjectName();
                array5[i] = array[i].getListener();
                array6[i] = array[i].getNotificationFilter();
                array7[i] = new MarshalledObject(array6[i]);
                array8[i] = array[i].getHandback();
            }
            try {
                Integer[] access$1500 = RMIConnector.access$1500(RMIConnector.this, array4, array7, array3, false);
                for (int j = 0; j < length; ++j) {
                    array2[j] = new ClientListenerInfo(access$1500[j], array4[j], array5[j], array6[j], array8[j], array3[j]);
                }
                RMIConnector.this.rmiNotifClient.postReconnection(array2);
                return;
            }
            catch (InstanceNotFoundException ex) {
                int n = 0;
                for (int k = 0; k < length; ++k) {
                    try {
                        array2[n++] = new ClientListenerInfo(RMIConnector.access$600(RMIConnector.this, array4[k], new MarshalledObject(array6[k]), array3[k], false), array4[k], array5[k], array6[k], array8[k], array3[k]);
                    }
                    catch (InstanceNotFoundException ex2) {
                        RMIConnector.logger.warning("reconnectNotificationListeners", "Can't reconnect listener for " + array4[k]);
                    }
                }
                if (n != length) {
                    array2 = new ClientListenerInfo[n];
                    System.arraycopy(array2, 0, array2, 0, n);
                }
                RMIConnector.this.rmiNotifClient.postReconnection(array2);
                return;
            }
        }
        
        @Override
        protected void checkConnection() throws IOException {
            if (RMIConnector.logger.debugOn()) {
                RMIConnector.logger.debug("RMIClientCommunicatorAdmin-checkConnection", "Calling the method getDefaultDomain.");
            }
            RMIConnector.this.connection.getDefaultDomain(null);
        }
        
        @Override
        protected void doStart() throws IOException {
            RMIServer rmiServer;
            try {
                rmiServer = ((RMIConnector.this.rmiServer != null) ? RMIConnector.this.rmiServer : RMIConnector.access$1900(RMIConnector.this, RMIConnector.this.jmxServiceURL, RMIConnector.this.env));
            }
            catch (NamingException obj) {
                throw new IOException("Failed to get a RMI stub: " + obj);
            }
            RMIConnector.this.connection = RMIConnector.connectStub(rmiServer, RMIConnector.this.env).newClient(RMIConnector.this.env.get("jmx.remote.credentials"));
            this.reconnectNotificationListeners(RMIConnector.this.rmiNotifClient.preReconnection());
            RMIConnector.this.connectionId = RMIConnector.this.getConnectionId();
            RMIConnector.access$1200(RMIConnector.this, new JMXConnectionNotification("jmx.remote.connection.opened", this, RMIConnector.this.connectionId, RMIConnector.this.clientNotifSeqNo++, "Reconnected to server", null));
        }
        
        @Override
        protected void doStop() {
            try {
                RMIConnector.this.close();
            }
            catch (IOException ex) {
                RMIConnector.logger.warning("RMIClientCommunicatorAdmin-doStop", "Failed to call the method close():" + ex);
                RMIConnector.logger.debug("RMIClientCommunicatorAdmin-doStop", ex);
            }
        }
    }
    
    private static final class ObjectInputStreamWithLoader extends ObjectInputStream
    {
        private final ClassLoader loader;
        
        ObjectInputStreamWithLoader(InputStream in, ClassLoader loader) throws IOException {
            super(in);
            this.loader = loader;
        }
        
        @Override
        protected Class<?> resolveClass(ObjectStreamClass objectStreamClass) throws IOException, ClassNotFoundException {
            String name = objectStreamClass.getName();
            ReflectUtil.checkPackageAccess(name);
            return Class.forName(name, false, this.loader);
        }
    }
}
